function [In_D,In_A,dt,dD,c_D,c_A,q_D,q_A,units_I,units_t] = generate_synthetic_data( ...
                                                        units_I, units_t,...
                                                        f,d,N,c_D,c_A,q_D,q_A,...
                                                        tht,rho_D,rho_A,kap_D,kap_A, ...
                                                        pm,ps, w0_D,w1_D,ws_D,w0_A,w1_A,ws_A,demo)

%% if no input, set default values
if nargin<1

    % units
    units_I = 'photon';  % units of intensity, i.e. [I]
    units_t = 's';       % units of time, i.e. [t]

    % frame rate
    f = 10/1;  % 1/[t]
    
    % fraction of dead time per measurement acquisition period (dt)
    d = 0.01;
    
    % total number of steps
    N = 1000;

    % cross-talk coefficients
    c_D = 0.90; % D -> D
    c_A = 0.75; % A -> A
    
    % quantum efficiency
    q_D = 0.85; % D
    q_A = 0.75; % A
    
    % overall emission rate
    tht = 1e3;  % [I]/[t]
    
    % background emission multipliers
    rho_D = 0.05;
    rho_A = 0.10;

    % molecule emission multipliers
    kap_S = 6;
    kap_D = [3.5 2.5 1.5];
    kap_A = [1.0 2.0 3.0];
    
    % transtion probs
    pm = [0.96  0.04   0.00
          0.04  0.92   0.04
          0.00  0.04   0.96];
    ps = [0.50  0.50   0.00];

    w0_D = 0.25;    % 0 -> 1
    w1_D = 0.98;    % 1 -> 1
    ws_D = 0.80;    % * -> 1
    
    w0_A = 0.15;    % 0 -> 1
    w1_A = 0.96;    % 1 -> 1
    ws_A = 0.90;    % * -> 1

    
    demo = true;
    
end


% total duration
T = N/f;  % [t]

% background emission rates
ksi_D = rho_D*tht;   % [I]/[t]
ksi_A = rho_A*tht;   % [I]/[t]

% molecule emission rates
lam_D = kap_D*tht;   % [I]/[t]
lam_A = kap_A*tht;   % [I]/[t]
lam_S = kap_S*tht;   % [I]/[t]


%% generate state traces
sn   = nan(1,N);
fn_D = nan(1,N);
fn_A = nan(1,N);

sn  (1) = sample(ps);
fn_D(1) = ( rand(1) <= ws_D );
fn_A(1) = ( rand(1) <= ws_A );

for n = 2:N

    sn(n) = sample( pm(sn(n-1),:) );

    if fn_D(n-1)
        fn_D(n) = ( rand(1) <= w1_D );
    else
        fn_D(n) = ( rand(1) <= w0_D );
    end
    
    if fn_A(n-1)
        fn_A(n) = ( rand(1) <= w1_A );
    else
        fn_A(n) = ( rand(1) <= w0_A );
    end
    
end

%% generate intensities

dt = T/N;       % [t]
dD = (1-d)*dt;  % [t]

In_D = ksi_D +    c_D *fn_D.*( fn_A.*lam_D(sn)+(1-fn_A)*lam_S ) + (1-c_A)*fn_D.*fn_A.*lam_A(sn);
In_A = ksi_A + (1-c_D)*fn_D.*( fn_A.*lam_D(sn)+(1-fn_A)*lam_S ) +    c_A *fn_D.*fn_A.*lam_A(sn);

In_D = poissrnd( q_D*dD*In_D );
In_A = poissrnd( q_A*dD*In_A );

In_D = max(In_D,realmin);
In_A = max(In_A,realmin);


%% show demo
if demo
    
    col_M = [0 0 1];   % main molecule color
    col_D = [0 1 0];   % main donor color
    col_A = [1 0 0];   % main acceptor color
    col_m = [0 1 1];   % aux molecule color


    M = length(lam_D);
    tn_bnd = (0:N)*dt;   % [t]
    
    subplot(5,1,1)
    h = plot_stairs(tn_bnd,sn);
    h(1).Color = col_M;
    ylabel({'State of','molecule'})
    ylim([0 M+1])
    set(gca,'YTick',1:M,'YTickLabel',[repmat('\sigma_',M,1),num2str((1:M)')])
    box off
    
    subplot(5,1,2)
    h = plot_stairs(tn_bnd,[fn_D+0.02;fn_A-0.02]);
    h(1).Color = col_D;
    h(2).Color = col_A;
    ylabel({'State of','fluorophores'})
    ylim([-0.5 1.5])
    set(gca,'YTick',[0 1],'YTickLabel',{'Dark','Bright'})
    legend('Donor','Acceptor','location','NorthEast','orientation','horizontal')
    legend boxoff
    box off
    
    subplot(5,1,[3 4])
    h = plot_stairs(tn_bnd,[In_D;In_A]);
    h(1).Color = col_D;
    h(2).Color = col_A;
    ylabel({'Intensities',['(',units_I,')']})
    legend('Donor channel','Acceptor channel','location','NorthEast','orientation','horizontal')
    legend boxoff
    box off
    
    subplot(5,1,5)
    h = plot_stairs(tn_bnd,[100*In_A./(In_D+In_A);100*lam_A(sn)./(lam_D(sn)+lam_A(sn))]);
    h(1).Color = col_M;
    h(2).Color = col_m;
    ylabel({'FRET','efficiency (%)'})
    xlabel(['Time (',units_t,')'])
    ylim([-20 150])
    legend('Apparent','True','location','NorthEast','orientation','horizontal')
    legend boxoff
    box off

end



if nargout<1
    clear In_D In_A units
end
















%% misc
function h = plot_stairs(r_edges,w_centers)

if nargin<1
    r_edges = [0 1 2.5 3 4]';

    w_centers = rand(length(r_edges)-1,5);
    
end

if isrow(r_edges)
    r_edges = r_edges';
end

if size(w_centers,1)+1 ~= length(r_edges)
    w_centers = w_centers';
end

h = stairs(r_edges,[w_centers;w_centers(end,:)]);
%h(end+1) = line(r_edges(2:end),w_centers,'marker','.','linestyle','none','color',h(1).Color);

% line(r_edges*[1 1],get(gca,'Ylim'),'color','k','linestyle',':')
% xlabel('r')
% ylabel('w')

if nargout<1
    clear h
end


